import Moleculer from 'moleculer';
import { Service, ServiceBroker, ServiceSchema } from 'moleculer';
import DbService, { DbServiceSettings } from 'moleculer-db';
import { CursorOptions, DbAdapter, QueryOptions } from 'moleculer-db';
import * as orientjs from 'orientjs';
export interface OrientDbServiceSchema<E, S extends Moleculer.ServiceSettingSchema = Moleculer.ServiceSettingSchema> extends ServiceSchema<S> {
    adapter: OrientDBAdapter<E>;
    settings?: S & DbServiceSettings;
    database: orientjs.DatabaseOptions;
    dataClass: {
        name: string;
        parentName?: string;
        isAbstract?: boolean;
        ifnotexist?: boolean;
        sequences?: {
            id: {
                name: string;
            };
        };
        indexes?: orientjs.IndexConfig[];
        properties?: {
            [name: string]: orientjs.PropertyCreateConfig;
        };
    };
}
export declare class OrientDBAdapter<E> implements DbAdapter {
    opts: any[];
    broker: ServiceBroker;
    service: DbService<E>;
    database: orientjs.ODatabaseSession;
    dataClass: orientjs.OClass;
    private sequences;
    client: orientjs.OrientDBClient;
    idField: string;
    /**
     * Creates an instance of OrientDBAdapter.
     * @param {any} opts
     *
     * @memberof OrientDBAdapter
     */
    constructor(...opts: any[]);
    /**
     * Initialize adapter
     *
     * @param {ServiceBroker} broker
     * @param {Service} service
     *
     * @memberof OrientDBAdapter
     */
    init(broker: ServiceBroker, service: Service): this;
    /**
     * Connect to database
     *
     * @returns {Promise}
     *
     * @memberof OrientDBAdapter
     */
    connect(): Promise<void>;
    /**
     * Disconnect from database
     *
     * @returns {Promise}
     *
     * @memberof OrientDBAdapter
     */
    disconnect(): Promise<void>;
    private getDatabase;
    /**
     * Find all entities by filters.
     *
     * Available filter props:
     *  - limit
     *  - offset
     *  - sort
     *  - search
     *  - searchFields
     *  - query
     *
     * @param {Object} filters
     * @returns {Promise<Array>}
     *
     * @memberof OrientDBAdapter
     */
    find<R, C = CursorOptions<R>>(filters?: C): Promise<R[]>;
    /**
     * Find an entity by query
     *
     * @param {Object} query
     * @returns {Promise}
     * @memberof OrientDBAdapter
     */
    findOne<R, Q = any>(query: Q): Promise<R>;
    /**
     * Find an entities by ID.
     *
     * @param {String} id
     * @returns {Promise<Object>} Return with the found document.
     *
     * @memberof OrientDBAdapter
     */
    findById<R>(id: string): Promise<R>;
    /**
     * Find any entities by IDs.
     *
     * @param {Array} idList
     * @returns {Promise<Array>} Return with the found documents in an Array.
     *
     * @memberof OrientDBAdapter
     */
    findByIds<R>(idList: Array<number | string>): Promise<R | R[]>;
    /**
     * Get count of filtered entites.
     *
     * Available query props:
     *  - search
     *  - searchFields
     *  - query
     *
     * @param {Object} [filters={}]
     * @returns {Promise<Number>} Return with the count of documents.
     *
     * @memberof OrientDBAdapter
     */
    count(filters?: any): Promise<number>;
    /**
     * Insert an entity.
     *
     * @param {Object} entity
     * @returns {Promise<Object>} Return with the inserted document.
     *
     * @memberof OrientDBAdapter
     */
    insert<T, R = T>(entity: T): Promise<R>;
    /**
     * Insert many entities
     *
     * @param {Array} entities
     * @returns {Promise<Array<Object>>} Return with the inserted documents in an Array.
     *
     * @memberof OrientDBAdapter
     */
    insertMany<T, R = T>(...entities: T[]): Promise<R[]>;
    /**
     * Update many entities by `query` and `update`
     *
     * @param {Object} query
     * @param {Object} update
     * @returns {Promise<Number>} Return with the count of modified documents.
     *
     * @memberof OrientDBAdapter
     */
    updateMany<T, Q extends QueryOptions<T> = any>(query: Q, update: T): Promise<number>;
    /**
     * Update an entity by ID and `update`
     *
     * @param {String} id - ObjectID as hexadecimal string.
     * @param {Object} update
     * @returns {Promise<Object>} Return with the updated document.
     *
     * @memberof OrientDBAdapter
     */
    updateById<T, R = T>(id: string | number, update: T): Promise<R>;
    /**
     * Remove entities which are matched by `query`
     *
     * @param {Object} query
     * @returns {Promise<Number>} Return with the count of deleted documents.
     *
     * @memberof OrientDBAdapter
     */
    removeMany<TQuery>(query: TQuery): Promise<number>;
    /**
     * Remove an entity by ID
     *
     * @param {String} id - ObjectID as string.
     * @returns {Promise<Object>} Return with the removed document.
     *
     * @memberof OrientDBAdapter
     */
    removeById<R>(id: string | number): Promise<R>;
    /**
     * Clear all entities from collection
     *
     * @returns {Promise}
     *
     * @memberof OrientDBAdapter
     */
    clear(): Promise<void>;
    /**
     * Convert DB entity to JSON object. It converts the `_id` to hexadecimal `String`.
     *
     * @param {Object} entity
     * @returns {Object}
     * @memberof OrientDBAdapter
     */
    entityToObject<T, R>(entity: T): R;
    /**
     * Create a filtered cursor.
     *
     * Available filters in `QueryOptions`:
     *    - search
     *    - sort
     *    - limit
     *    - offset
     *    - query
     *
     * @param {QueryOptions} params
     * @param {Boolean} isCounting
     * @returns {OQuery<T>}
     */
    createCursor<T = any, C extends CursorOptions<T> = any, Q = orientjs.OQuery<T>>(params?: C, isCounting?: boolean): Q;
    /**
     * Transforms 'idField' into OrientDB's '_id'
     * @param {Object} entity
     * @param {String} idField
     * @memberof OrientDBAdapter
     * @returns {Object} Modified entity
     */
    beforeSaveTransformID<T, R = T>(entity: T, idField: string): R;
    /**
     * Transforms OrientDB's 'id' into user defined 'idField'
     * @param {Object} entity
     * @param {String} idField
     * @memberof OrientDBAdapter
     * @returns {Object} Modified entity
     */
    afterRetrieveTransformID<T>(entity: T, idField: string): any;
    /**
     * Return OrientDB Client instance
     *
     * @returns {orientjs.OrientDBClient}
     */
    getClient(): orientjs.OrientDBClient;
    /**
     * Return orientjs.OClass instance
     *
     * @returns {orientjs.OClass}
     */
    getDataClass(): orientjs.OClass;
    /**
     * execute orientdb command.
     * @param command command
     * @param options options
     */
    runCommand<R>(command: string, options?: any): orientjs.OResult<R>;
    /**
     * execute orientdb query
     * @param query query
     * @param options options
     */
    runQuery<R>(query: string, options?: any): orientjs.OResult<R>;
    /**
     * execute orientdb live query for reactive query
     * @param query query
     * @param options options
     */
    runLive(query: string, options?: any): orientjs.LiveQuery;
}
