/* eslint-disable prettier/prettier */
/* eslint-disable @typescript-eslint/explicit-function-return-type */
/* eslint-disable @typescript-eslint/no-unused-vars */
/*
 * moleculer-db-adapter-orientdb
 * Copyright (c) 2019 Saeed Tabrizi (https://github.com/SaeedTabrizi/moleculer-db-adapter-orientdb)
 * MIT Licensed
 */
'use strict';
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const orientjs = __importStar(require("orientjs"));
const util_1 = require("util");
const utils_1 = require("./utils");
class OrientDBAdapter {
    /**
     * Creates an instance of OrientDBAdapter.
     * @param {any} opts
     *
     * @memberof OrientDBAdapter
     */
    constructor(...opts) {
        this.sequences = {};
        this.idField = 'id';
        this.opts = opts;
    }
    /**
     * Initialize adapter
     *
     * @param {ServiceBroker} broker
     * @param {Service} service
     *
     * @memberof OrientDBAdapter
     */
    init(broker, service) {
        this.broker = broker;
        this.service = service;
        this.service.schema.settings.idField = 'id';
        if (!this.service.schema.database) {
            /* istanbul ignore next */
            throw new Error('Missing `database` definition in schema of service!');
        }
        if (!this.service.schema.dataClass) {
            /* istanbul ignore next */
            throw new Error('Missing `dataClass` definition in schema of service!');
        }
        return this;
    }
    /**
     * Connect to database
     *
     * @returns {Promise}
     *
     * @memberof OrientDBAdapter
     */
    async connect() {
        const { database, dataClass } = this.service.schema;
        const dataClient = this.opts[0] || { host: 'localhost', port: 2424 };
        const serverCred = {
            username: dataClient.username,
            password: dataClient.password,
        };
        const dbOptions = { ...serverCred, ...database };
        try {
            this.broker.logger.info(`Connecting to orientdb on host: '${dataClient.host}' , port : '${dataClient.port}' , username: ${dataClient.username}' , password: ${dataClient.password}' `);
            this.client = await orientjs.OrientDBClient.connect(dataClient);
            const exists = await this.client.existsDatabase(dbOptions);
            if (!exists) {
                await this.client.createDatabase(dbOptions);
            }
            const sessions = await this.client.sessions(database);
            this.database = await sessions.acquire();
            // dbPool.once("close", () => {
            //   this.service.logger.warn("Disconnected from db");
            // });
            let cls;
            try {
                cls = await this.database.class.create(dataClass.name, dataClass.parentName, dataClass.cluster, dataClass.isAbstract, dataClass.ifnotexist);
                this.service.logger.info(`The Class '${cls.name}' has been created`);
            }
            catch (error) {
                this.service.logger.error(`The Class '${dataClass.name}' not be created`);
            }
            if (dataClass.properties && cls) {
                // tslint:disable-next-line:forin
                for (const key in dataClass.properties) {
                    try {
                        await cls.property.create({
                            name: key,
                            ...dataClass.properties[key],
                        });
                        this.service.logger.info(`The Property '${cls.name}.${key}' has been created`);
                    }
                    catch (error) {
                        this.service.logger.error(`The Property '${cls.name}.${key}' not be created`);
                    }
                }
            }
            if (dataClass.indexes) {
                for (const item of dataClass.indexes) {
                    try {
                        await this.database.index.create(item);
                    }
                    catch (error) {
                        this.service.logger.error(`The Index '${item.name}' not be created`);
                    }
                }
            }
            if (dataClass.sequences) {
                const ss = Object.entries(dataClass.sequences);
                for (const item of ss) {
                    const k = item[0];
                    const c = item[1];
                    let s; //
                    try {
                        s = await this.database.sequence.get(c.name);
                    }
                    catch (error) {
                        this.service.logger.info(`The Sequence '${c.name}' not found`);
                    }
                    if (!s) {
                        s = await this.database.sequence.create(c.name, 'ORDERED', 1);
                        this.service.logger.info(`The Sequence '${c.name}' has been created`);
                    }
                    if (s) {
                        this.sequences[k] = s;
                        this.service.logger.info(`The Sequence '${c.name}' has been attached`);
                    }
                }
            }
            this.service.logger.warn(`Connected to "${this.database.name}" db`);
            return;
        }
        catch (error) {
            this.service.logger.error('Error while connecting to db', error);
            return;
        }
    }
    /**
     * Disconnect from database
     *
     * @returns {Promise}
     *
     * @memberof OrientDBAdapter
     */
    disconnect() {
        if (this.client) {
            return this.client.close();
        }
        return Promise.resolve();
    }
    async getDatabase() {
        if (!this.database) {
            const { database } = this.service.schema;
            const sessions = await this.client.sessions(database);
            this.database = await sessions.acquire();
        }
        return this.database;
    }
    /**
     * Find all entities by filters.
     *
     * Available filter props:
     *  - limit
     *  - offset
     *  - sort
     *  - search
     *  - searchFields
     *  - query
     *
     * @param {Object} filters
     * @returns {Promise<Array>}
     *
     * @memberof OrientDBAdapter
     */
    async find(filters) {
        return this.createCursor(filters, false).all();
    }
    /**
     * Find an entity by query
     *
     * @param {Object} query
     * @returns {Promise}
     * @memberof OrientDBAdapter
     */
    async findOne(query) {
        return this.createCursor({ query }, false).one();
    }
    /**
     * Find an entities by ID.
     *
     * @param {String} id
     * @returns {Promise<Object>} Return with the found document.
     *
     * @memberof OrientDBAdapter
     */
    async findById(id) {
        const svc = this.service;
        try {
            return this.createCursor({ query: { [svc.schema.settings.idField]: id } }, false).one();
        }
        catch (error) {
            svc.logger.error(`Error occured in '${svc.schema.dataClass.name}' findByIds`, error);
            throw error;
        }
    }
    /**
     * Find any entities by IDs.
     *
     * @param {Array} idList
     * @returns {Promise<Array>} Return with the found documents in an Array.
     *
     * @memberof OrientDBAdapter
     */
    async findByIds(idList) {
        const svc = this.service;
        const db = svc && svc.adapter.database;
        try {
            if (Array.isArray(idList)) {
                return this.createCursor({ query: { [svc.schema.settings.idField]: { $in: idList } } }, false).all();
            }
            else {
                return this.createCursor({ query: { [svc.schema.settings.idField]: idList } }, false).one();
            }
        }
        catch (error) {
            svc.logger.error(`Error occured in '${svc.schema.dataClass.name}' findByIds`, error);
            throw error;
        }
    }
    /**
     * Get count of filtered entites.
     *
     * Available query props:
     *  - search
     *  - searchFields
     *  - query
     *
     * @param {Object} [filters={}]
     * @returns {Promise<Number>} Return with the count of documents.
     *
     * @memberof OrientDBAdapter
     */
    async count(filters) {
        const q = this.createCursor(filters, true);
        const c = await q.one();
        return c.count ? c.count : 0;
    }
    /**
     * Insert an entity.
     *
     * @param {Object} entity
     * @returns {Promise<Object>} Return with the inserted document.
     *
     * @memberof OrientDBAdapter
     */
    async insert(entity) {
        const svc = this.service;
        const db = svc && svc.adapter.database;
        const seqs = svc.schema.dataClass.sequences;
        try {
            // if (seqs) {
            //     for (const key in seqs) {
            //         if (seqs.hasOwnProperty(key)) {
            //             const name = seqs[key].name;
            //             entity[key] = db.rawExpression(`sequence('${name}').next()`);
            //         }
            //     }
            // }
            const r = await db
                .insert()
                .into(svc.schema.dataClass.name)
                .set(entity)
                .one();
            return r;
        }
        catch (error) {
            svc.logger.error(`Error occured in '${svc.schema.dataClass.name}' insert record`, error);
            throw error;
        }
    }
    /**
     * Insert many entities
     *
     * @param {Array} entities
     * @returns {Promise<Array<Object>>} Return with the inserted documents in an Array.
     *
     * @memberof OrientDBAdapter
     */
    async insertMany(...entities) {
        const svc = this.service;
        const db = svc && svc.adapter.database;
        try {
            // const s = await this.client.sessions();
            // const db = await s.acquire();
            const A = [];
            for (const entity of entities) {
                for (const key in this.sequences) {
                    if (this.sequences.hasOwnProperty(key)) {
                        const name = this.sequences[key].name;
                        entity[key] = db.rawExpression(`sequence('${name}').next()`);
                    }
                }
                const r = await db
                    .insert()
                    .into(svc.schema.dataClass.name)
                    .set(entity)
                    .one();
                A.push(r);
            }
            return A;
        }
        catch (error) {
            svc.logger.error(`Error occured in '${svc.schema.dataClass.name}' insert records`, error);
            throw error;
        }
    }
    /**
     * Update many entities by `query` and `update`
     *
     * @param {Object} query
     * @param {Object} update
     * @returns {Promise<Number>} Return with the count of modified documents.
     *
     * @memberof OrientDBAdapter
     */
    async updateMany(query, update) {
        const svc = this.service;
        const db = svc && svc.adapter.database;
        try {
            // const s = await this.client.sessions();
            // const db = await s.acquire();
            const r = await db
                .update(svc.schema.dataClass.name)
                .set(update)
                .where(query)
                .one();
            return r;
        }
        catch (error) {
            svc.logger.error(`Error occured in '${svc.schema.dataClass.name}' update records`, error);
            throw error;
        }
    }
    /**
     * Update an entity by ID and `update`
     *
     * @param {String} id - ObjectID as hexadecimal string.
     * @param {Object} update
     * @returns {Promise<Object>} Return with the updated document.
     *
     * @memberof OrientDBAdapter
     */
    async updateById(id, update) {
        const svc = this.service;
        const db = svc && svc.adapter.database;
        try {
            // const s = await this.client.sessions();
            // const db = await s.acquire();
            const m = update.$set;
            const r = await db
                .update(svc.schema.dataClass.name)
                .set(m)
                .where({ [svc.schema.settings.idField]: id })
                .return('AFTER')
                .one();
            return r;
        }
        catch (error) {
            svc.logger.error(`Error occured in '${svc.schema.dataClass.name}' update record`, error);
            throw error;
        }
    }
    /**
     * Remove entities which are matched by `query`
     *
     * @param {Object} query
     * @returns {Promise<Number>} Return with the count of deleted documents.
     *
     * @memberof OrientDBAdapter
     */
    async removeMany(query) {
        const svc = this.service;
        const db = svc && svc.adapter.database;
        try {
            // const s = await this.client.sessions();
            // const db = await s.acquire();
            const r = await db
                .delete()
                .from(svc.schema.dataClass.name)
                .where(query)
                .one();
            return r;
        }
        catch (error) {
            svc.logger.error(`Error occured in '${svc.schema.dataClass.name}' remove records`, error);
            throw error;
        }
    }
    /**
     * Remove an entity by ID
     *
     * @param {String} id - ObjectID as string.
     * @returns {Promise<Object>} Return with the removed document.
     *
     * @memberof OrientDBAdapter
     */
    async removeById(id) {
        const svc = this.service;
        const db = svc && svc.adapter.database;
        try {
            // const s = await this.client.sessions();
            // const db = await s.acquire();
            const r = await db
                .command(`DELETE  FROM ${svc.schema.dataClass.name} WHERE ${svc.settings.idField} = :id LIMIT 1 UNSAFE`, { params: { id } })
                .one();
            return { id };
        }
        catch (error) {
            svc.logger.error(`Error occured in '${svc.schema.dataClass.name}' remove record`, error);
            throw error;
        }
    }
    /**
     * Clear all entities from collection
     *
     * @returns {Promise}
     *
     * @memberof OrientDBAdapter
     */
    async clear() {
        const svc = this.service;
        const db = svc && svc.adapter.database;
        try {
            await db.command(`TRUNCATE CLASS ${svc.schema.dataClass.name} UNSAFE`).one();
        }
        catch (error) {
            svc.logger.error(`Error occured in '${svc.schema.dataClass.name}' clear records`, error);
            throw error;
        }
    }
    /**
     * Convert DB entity to JSON object. It converts the `_id` to hexadecimal `String`.
     *
     * @param {Object} entity
     * @returns {Object}
     * @memberof OrientDBAdapter
     */
    entityToObject(entity) {
        return Object.assign({}, entity);
    }
    /**
     * Create a filtered cursor.
     *
     * Available filters in `QueryOptions`:
     *    - search
     *    - sort
     *    - limit
     *    - offset
     *    - query
     *
     * @param {QueryOptions} params
     * @param {Boolean} isCounting
     * @returns {OQuery<T>}
     */
    createCursor(params, isCounting = false) {
        const svc = this.service;
        const db = svc && svc.adapter.database;
        try {
            if (!db) {
                throw new Error('Database not initialized');
            }
            const mfields = (params && params.fields) || (svc.schema.settings && svc.schema.settings.fields);
            const fields = typeof mfields === 'string' ? mfields.split(',') : mfields;
            if (params) {
                let q = db
                    .select(isCounting ? 'count(*) AS count' : fields ? fields.join(',') : '*')
                    .from(svc.schema.dataClass.name);
                // Filter
                if (params.search) {
                    let sfields = [];
                    if (params.searchFields) {
                        sfields =
                            typeof params.searchFields === 'string'
                                ? params.searchFields.split(',')
                                : params.searchFields;
                        const f = Object.fromEntries(sfields.map(xx => [xx, params.search]));
                        q = q.where(f);
                    }
                }
                else {
                    if (params.query) {
                        const s = utils_1.getQueryFromFilterExpression(params.query);
                        q = q.where(s);
                    }
                }
                // Sort
                if (params.sort) {
                    const srt = Array.isArray(params.sort) ? params.sort : [params.sort];
                    const sort = {};
                    srt.forEach(xx => {
                        if (xx.includes('-')) {
                            sort[xx] = 'desc';
                        }
                        else {
                            sort[xx] = 'asc';
                        }
                    });
                    q = q.order(sort);
                }
                // Paging
                if (params.limit || params.offset) {
                    if (util_1.isNumber(params.offset) && params.offset > 0) {
                        q = q.skip(params.offset);
                    }
                    if (util_1.isNumber(params.limit) && params.limit > 0) {
                        q = q.limit(params.limit);
                    }
                }
                return q;
            }
            return db
                .select(isCounting ? 'count(*) AS count' : fields ? fields.join(',') : '*')
                .from(svc.schema.dataClass.name);
        }
        catch (error) {
            svc.logger.error(`Error occured in '${svc.schema.dataClass.name}' createCursor`, error);
            throw error;
        }
    }
    /**
     * Transforms 'idField' into OrientDB's '_id'
     * @param {Object} entity
     * @param {String} idField
     * @memberof OrientDBAdapter
     * @returns {Object} Modified entity
     */
    beforeSaveTransformID(entity, idField) {
        return entity;
    }
    /**
     * Transforms OrientDB's 'id' into user defined 'idField'
     * @param {Object} entity
     * @param {String} idField
     * @memberof OrientDBAdapter
     * @returns {Object} Modified entity
     */
    afterRetrieveTransformID(entity, idField) {
        // if (idField !== "_id") {
        // 	entity[idField] = entity["_id"];
        // 	delete entity._id;
        // }
        delete entity['@class'];
        delete entity['@version'];
        delete entity['@rid'];
        delete entity['@type'];
        return entity;
    }
    /**
     * Return OrientDB Client instance
     *
     * @returns {orientjs.OrientDBClient}
     */
    getClient() {
        return this.client;
    }
    /**
     * Return orientjs.OClass instance
     *
     * @returns {orientjs.OClass}
     */
    getDataClass() {
        return this.dataClass;
    }
    /**
     * execute orientdb command.
     * @param command command
     * @param options options
     */
    runCommand(command, options) {
        return this.database.command(command, options);
    }
    /**
     * execute orientdb query
     * @param query query
     * @param options options
     */
    runQuery(query, options) {
        return this.database.query(query, options);
    }
    /**
     * execute orientdb live query for reactive query
     * @param query query
     * @param options options
     */
    runLive(query, options) {
        return this.database.liveQuery(query, options);
    }
}
exports.OrientDBAdapter = OrientDBAdapter;
